@@ -12,7 +12,16 @@ import pytest
 @pytest.fixture(scope='session')
 def wheels_and_eggs(tmpdir_factory):
     """Build wheels and eggs from test distributions."""
-    test_distributions = "complex-dist", "simple.dist", "headers.dist", "unicode.dist"
+    test_distributions = "complex-dist", "simple.dist", "headers.dist"
+    if sys.version_info >= (3, 6):
+        # Only Python 3.6+ can handle packaging unicode file names reliably
+        # across different platforms
+        test_distributions += ("unicode.dist",)
+
+    if sys.platform != 'win32':
+        # ABI3 extensions don't really work on Windows
+        test_distributions += ("abi3extension.dist",)
+
     pwd = os.path.abspath(os.curdir)
     this_dir = os.path.dirname(__file__)
     build_dir = tmpdir_factory.mktemp('build')
